// Device USB Test.cpp : Defines the entry point for the console application.
//
// This application opens the USB ActiveSync Com Driver for communication with a Desktop PC.
// It reads 40 DWORDs and then sends 40 DWORDs, from values (0 - 39).
//
// ADS# 720020-12111

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>

int _tmain(int argc, _TCHAR* argv[])
{
	HANDLE hUsbPort = NULL;

	// USB COM Driver Name, can be different for different platforms
	WCHAR portName[] = TEXT("COM4:");
	
	DWORD buffer = 0;
	DWORD bytesRead = 0;
	DWORD bytesWritten = 0;
	DWORD eventBuffer;
	DWORD errorBuffer;
	
	// COM State Configuration Structure
	DCB state;

	// Comstats Structure, used to hold data from ClearCommError function
	COMSTAT comstats;

	int returnVal;
	int i = 0;
	char keyPress;

	wprintf(TEXT("Device USB Test   ADS# 720020-12111\r\n"));
	RETAILMSG(1, (TEXT("Device USB Test   ADS# 720020-12111\r\n")));

	wprintf(TEXT("Opening USB Port %s ...\r\n"), portName);
	RETAILMSG(1, (TEXT("Opening USB Port %s ...\r\n"), portName));

	// Open the USB Com Driver
	hUsbPort = CreateFile(	portName, 
							GENERIC_WRITE | GENERIC_READ, 
							0, 
							NULL, 
							OPEN_EXISTING, 
							FILE_ATTRIBUTE_NORMAL, 
							NULL);

	if (hUsbPort == INVALID_HANDLE_VALUE) {
		RETAILMSG(1, (TEXT("Could not open port %s\r\n"), portName));
		RETAILMSG(1, (TEXT("GetLastError returned %d\r\n"), GetLastError()));
		wprintf(TEXT("Could not open port %s\r\n"), portName);
		wprintf(TEXT("GetLastError returned %d"), GetLastError());

		Sleep(2000);		
		return -1;
	}
	else
	{
		wprintf(TEXT("... Opened USB Port %s\r\n"), portName);
		RETAILMSG(1, (TEXT("... Opened USB Port %s\r\n"), portName));
	}
	
	// Configure the DCB stucture for the COM driver
	// Settings of importance:
	// No flow control,
	// Max baudrate,
	// 8 bit data, no parity, 1 stop bit
	state.DCBlength = sizeof(DCB);
	state.BaudRate = CBR_256000;
	state.fBinary = TRUE;
	state.fOutxCtsFlow = FALSE;
	state.fOutxDsrFlow = FALSE;
	state.fDtrControl = DTR_CONTROL_DISABLE;
	state.fRtsControl = RTS_CONTROL_DISABLE;
	state.fDsrSensitivity = FALSE;
	state.fTXContinueOnXoff = FALSE;
	state.fOutX = FALSE;
	state.fInX = FALSE;
	state.fErrorChar = FALSE;
	state.fNull = FALSE;
	state.fAbortOnError = FALSE;
	state.wReserved = 0;
	state.ByteSize = 8;
	state.Parity = NOPARITY;
	state.StopBits = ONESTOPBIT;
	state.XonLim = 1;
	state.XoffLim = 1;

	// Set Comm State
	SetCommState(hUsbPort, &state);

	// Purge the input buffer
	PurgeComm(hUsbPort, PURGE_RXCLEAR);

	// Set Comm Mask to trigger upon recieving characters
	SetCommMask(hUsbPort, EV_RXCHAR);
	
	// Read 40 DWORDs (looking for the sent value of 39)
	while(buffer < 39)
	{
		// Wait for CommEvent on Com Port
		WaitCommEvent(hUsbPort, &eventBuffer, NULL);

		returnVal = ReadFile(hUsbPort, &buffer, sizeof(buffer), &bytesRead, NULL);
		
		// Check returnVal
		if(returnVal != 0)
		{
			RETAILMSG(1, (TEXT("%d) Bytes Read (%d): 0x%x\r\n"), i, bytesRead, buffer));
			wprintf(TEXT("%d) Bytes Read (%d): 0x%x\r\n"), i, bytesRead, buffer);
			i++;
		}
		else
		{
			RETAILMSG(1, (TEXT("Read Error on port %s\r\n"), portName));
			RETAILMSG(1, (TEXT("GetLastError returned %d\r\n"), GetLastError()));
			wprintf(TEXT("Read Error on port %s\r\n"), portName);
			wprintf(TEXT("GetLastError returned %d\r\n"), GetLastError());
		}
		
		// Check for additional data in queue
		ClearCommError(hUsbPort, &errorBuffer, &comstats);

		// While bytes still left in the queue
		while(comstats.cbInQue != 0)
		{
			returnVal = ReadFile(hUsbPort, &buffer, sizeof(buffer), &bytesRead, NULL);
			
			// Check returnVal
			if(returnVal != 0)
			{
				RETAILMSG(1, (TEXT("++ %d) Data Read (%d): 0x%x\r\n"), i, bytesRead, buffer));
				wprintf(TEXT("++ %d) Data Read (%d): 0x%x\r\n"), i, bytesRead, buffer);
				i++;
			}
			else
			{
				RETAILMSG(1, (TEXT("++ Read Error on port %s\r\n"), portName));
				RETAILMSG(1, (TEXT("++ GetLastError returned %d\r\n"), GetLastError()));
				wprintf(TEXT("++ Read Error on port %s\r\n"), portName);
				wprintf(TEXT("++ GetLastError returned %d\r\n"), GetLastError());
			}
			
			// Check for additonal data in queue
			ClearCommError(hUsbPort, &errorBuffer, &comstats);
		}
	}
	
	RETAILMSG(1, (TEXT("\r\n")));
	wprintf(TEXT("\r\n"));

	// Write 40 DWORDs
	for(buffer = 0; buffer < 40; buffer++)
	{
		wprintf(TEXT("%d)"), buffer);
		returnVal = WriteFile(hUsbPort, &buffer, sizeof(buffer), &bytesWritten, NULL);
			
		// Check returnVal
		if(returnVal != 0)
		{
			printf("Data Written (%d): 0x%x\r\n", bytesWritten, buffer);
			RETAILMSG(1, (TEXT("Data Written (%d): 0x%x\r\n"), bytesWritten, buffer));
		}
		else
		{
			wprintf(TEXT("Write Error on port %s\r\n"), portName);
			wprintf(TEXT("GetLastError returned %d\r\n"), GetLastError());
		}
	}

	wprintf(TEXT("Press ENTER to quit...\r\n"));
	wscanf(TEXT("%c"), &keyPress);

	return 0;
}